<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Rhawn\Services;

use App\Rhawn\Repositories\Contracts\RhawnProductRepository;
use App\Rhawn\Repositories\Eloquent\RhawnCustomerRepositoryEloquent;
use App\Rhawn\Repositories\Eloquent\RhawnRawRepositoryEloquent;
use App\Support\Facades\SimpleLogs;
use Illuminate\Support\Facades\DB;

class RhawnProductService
{
    public function __construct(RhawnProductRepository $rhawnProductRepository)
    {
        $this->rhawnRawRepository = app(RhawnRawRepositoryEloquent::class);
        $this->rhawnProductRepository = $rhawnProductRepository;
    }

    public function getRawsList($offset = 0, $limit = 100)
    {
        $rawList = $this->rhawnRawRepository->getRawListLimit($limit, $offset);
        return $rawList;
    }

    public function getChemicalPackage($rawIds)
    {
        $packageList = $this->rhawnRawRepository->getProductPackage($rawIds);
        return $packageList;
    }

    public function getProductPackagesThroughGroupByPackUnit()
    {
        return $this->rhawnRawRepository->getProductPackagesThroughGroupByPackUnit();
    }

    public function getChemicalRawThroughtCode($rawCode)
    {
        return $this->rhawnRawRepository->getRawProductsThroughtCode($rawCode);
    }

    public function getProductsThroughPid($pId)
    {
        try{
            $dbConnect = DB::connection('rhawn_mysql');

            $product = $dbConnect->table('products')
                ->where('p_id',$pId)
                ->get()->toArray();
            if(count($product) > 0){
                return current($product);
            }

            return null;

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':getRawProductsThroughPid ','error');
            throw $exception;
        }
    }

    /**
     * 通过产品code查找
     * @param $pCode
     */
    public function getProductsInfoByCode($pCode)
    {
        $productInfo = $this->rhawnProductRepository->getProductByPcode($pCode);
    }

    /**
     * 获取产品列表(含促销价格)
     * @param $offset
     * @param $pageSize
     * @param array $where
     * @return null
     */
    public function getProductsListWithPromotions($offset,$limit,$where = [])
    {
        try{
            $products = $this->rhawnProductRepository->getProductsList($where,$limit,$offset);
            if($products){
                $pids = [];
                foreach($products as $product){
                    $pids[] = $product['p_id'];
                }
                $promotions = $this->rhawnProductRepository->getProductPromotionByPid($pids);
                $productPromotions = [];
                if($promotions){
                    foreach($promotions as $promotion){
                        $productPromotions[$promotion->p_id] = $promotion;
                    }
                }
                foreach($products as &$product){
                    $product['m_price'] = $product['p_price'];
                    if(!empty($productPromotions) && isset($productPromotions[$product['p_id']])){
                        $pPromotions = $productPromotions[$product['p_id']];
                        if($pPromotions->pt_discount != 0){
                            $product['p_price'] = round($pPromotions->pt_discount * $product['p_price']);
                        }
                    }
                }
            }
            return $products;
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':getProductsListWithPromotions ','error');
            return null;
        }
    }

    /**
     * 根据CAS号获取产品(含促销价格)
     * @param $cas
     * @return false|mixed|null
     */
    public function getProductsWithPromotionsByCas($cas)
    {
        try{
            $products = $this->rhawnProductRepository->getProductsByCas($cas);
            if($products){
                $products = current($products);
                $promotions = $this->rhawnProductRepository->getProductPromotionByPid($products['p_id']);
                $productPromotions = [];
                if($promotions){
                    foreach($promotions as $promotion){
                        $productPromotions[$promotion->p_id] = $promotion;
                    }
                }
                $products['m_price'] = $products['p_price'];
                if(!empty($productPromotions) && isset($productPromotions[$products['p_id']])){
                    $pPromotions = $productPromotions[$products['p_id']];
                    if($pPromotions->pt_discount != 0){
                        $products['p_price'] = round($pPromotions->pt_discount * $products['p_price']);
                    }
                }
            }
            return $products;
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':getProductsWithPromotionsByCas ','error');
            return null;
        }
    }

    /**
     * 获取产品详情(含促销价格及会员价格)
     * @param $pCode
     * @param $cusCode
     * @return false|mixed|null
     */
    public function getProductDetailWithPromotions($pCode,$cusCode)
    {
        try{
            $products = $this->rhawnProductRepository->getProductByPcode($pCode);
            if($products){
                $products = current($products);
                $discount = app(RhawnCustomerRepositoryEloquent::class)->getCustomerDiscountByCusCode($cusCode);
                if($discount){
                    $products['m_price'] = $products['p_price'];
                    if(isset($discount[$products['b_id']])){
                        $products['p_price'] = round($discount[$products['b_id']]['cd_discount'] * $products['p_price']);
                    }else{
                        $products['p_price'] = $products['p_price'];
                    }
                }
                $promotions = $this->rhawnProductRepository->getProductPromotionByPid($products['p_id']);
                $productPromotions = [];
                if($promotions){
                    foreach($promotions as $promotion){
                        $productPromotions[$promotion->p_id] = $promotion;
                    }
                }
                if(!empty($productPromotions) && isset($productPromotions[$products['p_id']])){
                    $pPromotions = $productPromotions[$products['p_id']];
                    if($pPromotions->pt_discount != 0){
                        $products['p_price'] = round($pPromotions->pt_discount * $products['p_price']);
                    }
                }
            }
            return $products;
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':getProductDetailWithPromotions ','error');
            return null;
        }
    }
}
